@@ -232,11 +232,29 @@ imdb_languages = [
 ('pt','Português'),
 ]
 
+amazon_sites = [
+('com','US'),
+('co.uk','UK'),
+('ca','Canada'),
+('cn','China'),
+('fr','France'),
+('de','Germany'),
+('co.jp','Japan'),
+]
+
+music_types = [
+('artist','Artist'),
+('release','Release'),
+('track','Track'),
+]
+
 class Languages(object):
 
     def __init__(self):
         self.imdb = imdb_languages
         self.wikipedia = wikipedia_languages
+        self.amazon = amazon_sites
+        self.music = music_types
         self.web = self.news = self.weather = languages
         self.gweb = glanguages
         self.gnews = gnews_editions
@@ -251,5 +269,5 @@ class Languages(object):
             for (shorty,name) in lang_list:
                 if short == shorty:
                     return (shorty,name)
-        else: 
+        else:
             return lang_list
